@@ -14,10 +14,11 @@ int parse_res_msg(nt_res_tp resp, size_t colmns)
 {
 	int i, len, start, lines;
 	wchar_t ch;
-	wchar_t *buf, *cptr;
-	int offset, extend_ch, buflen;
+	wchar_t *buf, *cptr, *srcp;
+	int offset;
 	nt_link_tp linkp;
 	int consume_colmns;
+	int buf_idx;
 
 	assert(resp->msg);
 
@@ -31,85 +32,70 @@ int parse_res_msg(nt_res_tp resp, size_t colmns)
 	buf = malloc(sizeof(wchar_t) * (len + 11));
 	if(!buf)
 		return 0;
-	buflen = len+11;
-	wcscpy(buf, resp->msg);
+	buf_idx = 0;
+
+	srcp = resp->msg;
 
 	lines = 0;
-	extend_ch = 0;	
 	start = 0;
 	consume_colmns = 0;
 	for(i = 0; i < len; i++){
-		ch = buf[i];
+		ch = srcp[i];
 		switch(ch){
 		case L'&':
 			if(len > (i+5)){
-				if(buf[i+1] == L'q' &&
-					buf[i+2] == L'u' &&
-					buf[i+3] == L'o' &&
-					buf[i+4] == L't' &&
-					buf[i+5] == L';'){
-					if(nt_w_str_move(
-							buf+i, buflen-i, 6, -5)){
-						buf[i] = L'\"';
-						len -= 5;
-						break;
-					}else{
-						goto END_FOR;
-					}
+				if(srcp[i+1] == L'q' &&
+					srcp[i+2] == L'u' &&
+					srcp[i+3] == L'o' &&
+					srcp[i+4] == L't' &&
+					srcp[i+5] == L';'){
+					ch = L'\"';
+					buf[buf_idx] = ch;
+					buf_idx++;
+					i += 5;
 					break;
 				}
 			}
 			if(len > (i+4)){
-				if(buf[i+1] == L'a' &&
-					buf[i+2] == L'm' &&
-					buf[i+3] == L'p' &&
-					buf[i+4] == L';'){
-					if(nt_w_str_move(
-							buf+i, buflen-i, 5, -4)){
-						buf[i] = L'&';
-						len -= 4;
-						break;
-					}else{
-						goto END_FOR;
-					}
+				if(srcp[i+1] == L'a' &&
+					srcp[i+2] == L'm' &&
+					srcp[i+3] == L'p' &&
+					srcp[i+4] == L';'){
+					ch = L'&';
+					buf[buf_idx] = ch;
+					buf_idx++;
+					i += 4;
 					break;
 				}
 			}
 			if(len > (i+3)){
-				if(buf[i+1] == L'g' &&
-					buf[i+2] == L't' &&
-					buf[i+3] == L';'){
-					if(nt_w_str_move(
-							buf+i, buflen-i, 4, -3)){
-						buf[i] = L'>';
-						len -= 3;
-						break;
-					}else{
-						goto END_FOR;
-					}
+				if(srcp[i+1] == L'g' &&
+					srcp[i+2] == L't' &&
+					srcp[i+3] == L';'){
+					ch = L'>';
+					buf[buf_idx] = ch;
+					buf_idx++;
+					i += 3;
 					break;
-				}else if(buf[i+1] == L'l' &&
-					buf[i+2] == L't' &&
-					buf[i+3] == L';'){
-					if(nt_w_str_move(
-							buf+i, buflen-i, 4, -3)){
-						buf[i] = L'<';
-						len -= 3;
-						break;
-					}else{
-						goto END_FOR;
-					}
+				}else if(srcp[i+1] == L'l' &&
+					srcp[i+2] == L't' &&
+					srcp[i+3] == L';'){
+					ch = L'<';
+					buf[buf_idx] = ch;
+					buf_idx++;
+					i += 3;
 					break;
 				}
 			}
 			
 			goto DEF_LABEL; 
 		case L'<': 
-			if(len > (i+3) && buf[i+1] == L'b' &&
-					buf[i+2] == L'r' &&
-					buf[i+3] == L'>'){
+			if(len > (i+3) && srcp[i+1] == L'b' &&
+					srcp[i+2] == L'r' &&
+					srcp[i+3] == L'>'){
 				cptr = buf + start;
-				buf[i] = L'\0';
+				buf[buf_idx] = L'\0';
+				buf_idx++;
 				linkp = nt_link_add_data(
 						resp->msg_line_linkp, cptr);
 				if(!linkp)
@@ -119,70 +105,55 @@ int parse_res_msg(nt_res_tp resp, size_t colmns)
 
 				consume_colmns = 0;
 				lines++;
-				start = i + 4;
+				start = buf_idx;
 				i += 3;
 				break;
-			}else if(len > (i+7) && buf[i+1] == L'a' &&
-					buf[i+2] == L' ' &&
-					buf[i+3] == L'h' &&
-					buf[i+4] == L'r' &&
-					buf[i+5] == L'e' &&
-					buf[i+6] == L'f' &&
-					buf[i+7] == L'='){
-				cptr = wcschr(buf+i+8, L'>');
+			}else if(len > (i+7) && srcp[i+1] == L'a' &&
+					srcp[i+2] == L' ' &&
+					srcp[i+3] == L'h' &&
+					srcp[i+4] == L'r' &&
+					srcp[i+5] == L'e' &&
+					srcp[i+6] == L'f' &&
+					srcp[i+7] == L'='){
+				cptr = wcschr(srcp+i+8, L'>');
 				if(cptr){
-					offset = (cptr + 1) - &(buf[i]);
-					if(nt_w_str_move(
-							buf+i, buflen-i, offset, -1*offset)){
-						len -= offset;
-						i--;
-						break;
-					}else{
-						goto END_FOR;
-					}
-				}
-			}else if(len > (i+3) && buf[i+1] == L'/' &&
-					buf[i+2] == L'a' &&
-					buf[i+3] == L'>'){
-				if(nt_w_str_move(
-						buf+i, buflen-i, 4, -4)){
-					len -= 4;
-					i--;
+					offset = cptr - &(srcp[i]);
+					i += offset;
 					break;
-				}else{
-					goto END_FOR;
 				}
+			}else if(len > (i+3) && srcp[i+1] == L'/' &&
+					srcp[i+2] == L'a' &&
+					srcp[i+3] == L'>'){
+				i += 3;
+				break;
 			}
-		// fall through
+		/* fall through */
 		default:
 DEF_LABEL:
-			if(ch > 128)
-				offset = 2;
-			else
+			if(ch <= 128 || (ch >= 0xff66 && ch <= 0xff9d))
 				offset = 1;
+			else
+				offset = 2;
 
 			if(consume_colmns + offset > colmns){
-				if(nt_w_str_move(buf+i, buflen-i, 0, 1)){
-					cptr = buf + start;
-					buf[i] = L'\0';
-					linkp = nt_link_add_data(
-							resp->msg_line_linkp, cptr);
-					if(!linkp)
-						goto END_FOR;
-					if(resp->msg_line_linkp == NULL)
-						resp->msg_line_linkp = linkp;
+				cptr = buf + start;
+				buf[buf_idx] = L'\0';
+				linkp = nt_link_add_data(
+						resp->msg_line_linkp, cptr);
+				if(!linkp)
+					goto END_FOR;
+				if(resp->msg_line_linkp == NULL)
+					resp->msg_line_linkp = linkp;
 					
-					consume_colmns = 0;
-					lines++;
-					start = i + 1;
-				}else{
-					goto END_FOR;
-				}
-				extend_ch++;
-				if(extend_ch == 10)
-					goto END_FOR;
+				consume_colmns = 0;
+				lines++;
+				buf_idx++;
+				start = buf_idx;
+				i--;
 			}else{
 				consume_colmns += offset;
+				buf[buf_idx] = ch;
+				buf_idx++;
 			}
 			break;
 		}/* end switch */
@@ -190,7 +161,7 @@ DEF_LABEL:
 END_FOR:
 	if(consume_colmns > 0){
 		cptr = buf + start;
-		buf[i] = L'\0';
+		buf[buf_idx] = L'\0';
 		linkp = nt_link_add_data(
 				resp->msg_line_linkp, cptr);
 		if(linkp){
