@@ -4,9 +4,12 @@
 #include "net/nt_cookie.h"
 
 #define SSL_FLAG 1
-#define SET_FLAG(a,b) ((a)->flag  |= (b))
-#define CLR_FLAG(a,b) ((a)->flag  &= ~(b))
-#define IS_SET_FLAG(a,b) ((a)->flag & (b))
+#define GZIP_FLAG 2
+#define CHUNKED_FLAG 4
+#define RANGE_FLAG 8
+#define SET_FLAG(a,b) ((a)->flags  |= (b))
+#define CLR_FLAG(a,b) ((a)->flags  &= ~(b))
+#define IS_SET_FLAG(a,b) ((a)->flags & (b))
 
 #define HTTP_RESPONCE_HEADER_BUFFER_LEN  256
 
@@ -22,7 +25,7 @@ typedef struct tag_nt_http_header_t {
 	char *param;
 	char *last_modified;
 	size_t fsize;
-	unsigned int flag;
+	unsigned int flags;
 }nt_http_header_t;
 
 
@@ -43,7 +46,7 @@ extern BOOL nt_http_post(const char *url,
 		nt_link_tp headers, nt_cookie_tp cookiep);
 extern BOOL nt_http_get(const char *url, const char *out_path,
 		const char *referer, const char *user_agent, 
-		nt_link_tp headers);
+		nt_link_tp headers, BOOL range);
 extern int nt_http_parse_response_header(int readfd,
 		nt_http_response_header_tp responsep,
 		nt_cookie_tp cookeip, const char *host);
