@@ -10,5 +10,12 @@ extern nt_socket_tp nt_socket_init(int port, char *addr);
 extern void nt_socket_free(nt_socket_tp socketp);
 extern int nt_socket_connect(nt_socket_tp socketp, char* data, int data_len);
 
+extern BOOL nt_ssl_connect(
+	const char *connect_host_string,
+	const char *pem_path,
+	BOOL strict_verify,
+	char *wr_buf,
+	size_t *data_lenp,
+	size_t buf_len);
 
 #endif /* _NT_SOCKET_H_ */
