@@ -6,6 +6,8 @@
 #define	DISP_STATE_BOARDMENU	1
 #define	DISP_STATE_THREADTITLE	2
 #define	DISP_STATE_RESLIST		3
+#define	DISP_STATE_EDITOR		4 
+#define	DISP_STATE_HTML_RESULT	5
 
 
 extern int disp_board_menu(nt_window_tp wp, 
@@ -17,6 +19,8 @@ extern int disp_reslist(nt_window_tp wp,
 extern void free_board_menu_ctx(void *ptr);
 extern void free_threadlist_ctx(void *ptr);
 extern void free_reslist_ctx(void *ptr);
+extern BOOL disp_editor(nt_write_data_tp writep);
+extern int disp_html_result(nt_write_data_tp writep);
 
 #endif /* _DISP_H_ */
 
