@@ -9,6 +9,7 @@
 
 typedef int BOOL;
 typedef void (*nt_memfree_tp)(void* ptr);
+typedef BOOL (*nt_compare_tp)(void *lhs, void *rhs);
 
 typedef struct tag_nt_link_t *nt_link_tp;
 typedef struct tag_nt_link_t{
@@ -36,10 +37,12 @@ extern nt_link_tp nt_link_add_data(nt_link_tp link, void *data);
 extern nt_link_tp nt_link_remove(nt_link_tp src, nt_link_tp target);
 extern nt_link_tp nt_link_remove_by_data(nt_link_tp link, void *data);
 extern void* nt_link_get(nt_link_tp link);
-extern nt_link_tp nt_link_find(nt_link_tp link, void *data);
+extern nt_link_tp nt_link_find(nt_link_tp link, void *data, 
+			nt_compare_tp comp_func);
 extern nt_link_tp nt_link_next(nt_link_tp link);
 extern int nt_link_num(nt_link_tp linkp);
 extern void* nt_link_get_by_index(nt_link_tp linkp, int index);
+extern void nt_all_link_free(nt_link_tp ptr, nt_memfree_tp free_func);
 
 typedef struct tag_nt_key_value_t *nt_key_value_tp;
 typedef struct tag_nt_key_value_t{
@@ -47,6 +50,6 @@ typedef struct tag_nt_key_value_t{
 	char *value;
 }nt_key_value_t;
 
-extern nt_key_value_tp nt_kay_vlalue_alloc(char *key, char *value);
+extern nt_key_value_tp nt_key_value_alloc(char *key, char *value);
 
 #endif /* _NT_STD_T_H_ */
