@@ -5,6 +5,10 @@ extern BOOL nt_w_str_move(wchar_t *buf,
 			size_t buf_len,int move_index, int offset);
 extern wchar_t *nt_w_trim(const wchar_t *source);
 extern wchar_t *nt_w_trim_quotes(const wchar_t *source);
-extern char *nt_trim(const char *source);
+extern char *nt_trim2(const char *source, char *buf, size_t buf_len);
+#define nt_trim(a) nt_trim2((a), NULL,0)
+
+extern char* nt_str_clone(const char *cptr);
+extern char* nt_substr(const char *cptr, int start, int end);
 
 #endif /* _TEXT_H_ */
