@@ -157,8 +157,11 @@ static BOOL DoLoop(WINDOW *scrp)
 					ch = NT_KEY_NONE;
 					continue;
 				}else if(DISP_STATE_RESLIST == state){
-					if(!nt_read_thread(app_2ch_modelp))
-						break;
+					if(!nt_read_thread(app_2ch_modelp)){
+						wclear(scrp);
+						ch = NT_KEY_NONE;
+						continue;
+					}
 					if(rwinp->data){
 						free_reslist_ctx(rwinp->data);
 						rwinp->data = NULL;
@@ -173,7 +176,7 @@ static BOOL DoLoop(WINDOW *scrp)
 			werase(rwinp->wp);
 			title = app_2ch_modelp->selected_threadp->name;
 			num = draw_title(scrp, title, WA_REVERSE);
-			if(!nt_disp_win_move(scrp, twinp, 
+			if(!nt_disp_win_move(scrp, rwinp, 
 						LINES - num, COLS, num, 0))
 				goto END_WHILE;
 			rwinp->key = ch;
