@@ -12,6 +12,7 @@
 
 #include "utils/nt_std_t.h"
 #include "utils/text.h"
+#include "utils/zip.h"
 #include "utils/db.h"
 #include "net/nt_socket.h"
 #include "net/nt_http.h"
@@ -207,19 +208,22 @@ ERROR_TRAP:
 
 BOOL nt_http_get(const char *url, const char *out_path,
 		const char *referer, const char *user_agent, 
-		nt_link_tp extend_headers)
+		nt_link_tp extend_headers, BOOL range)
 {
 	nt_socket_tp socketp;
 	int fd = -1, sockfd;
 	char data[1024*2];
+	char itoa_buf[32];
 	char ssl_connect[512];
 	size_t data_len;
 	int nread, nwrite;
-	BOOL result;
+	BOOL result, retry;
 	nt_http_response_header_tp responsep;
 	nt_key_value_tp kv;
 	nt_link_tp linkp;
 
+	retry = FALSE;
+
 	nt_http_header_tp headerp = nt_http_init_header(url);
 	if(headerp == NULL)
 		return FALSE;
@@ -239,10 +243,19 @@ BOOL nt_http_get(const char *url, const char *out_path,
 		strcat(data, "\r\nReferer: ");
 		strcat(data, referer);
 	}
+	if(IS_SET_FLAG(headerp, RANGE_FLAG) && range){
+		if(0 <= sprintf(itoa_buf, "%lu-", headerp->fsize)){
+			strcat(data, "\r\nRange: bytes=");
+			strcat(data, itoa_buf);
+		}
+	}
 	if(headerp->last_modified){
 		strcat(data, "\r\nIf-Modified-Since: ");
 		strcat(data, headerp->last_modified);
 	}
+	if(!IS_SET_FLAG(headerp,SSL_FLAG) && 
+			!IS_SET_FLAG(headerp, RANGE_FLAG))
+		strcat(data, "\r\nAccept-Encoding: gzip");
 	if(extend_headers){
 		linkp = extend_headers;
 		do{
@@ -300,10 +313,9 @@ BOOL nt_http_get(const char *url, const char *out_path,
 		}
 	}
 
+	result = FALSE;
 	switch(responsep->status_code){
 	case 200:
-		nt_http_save_response_header(out_path, responsep);
-
 		fd = open(out_path,
 			O_CREAT | O_WRONLY,
 			S_IRUSR | S_IWUSR | S_IROTH);
@@ -312,6 +324,43 @@ BOOL nt_http_get(const char *url, const char *out_path,
 			fd = -1;
 			break;
 		}
+		if(IS_SET_FLAG(responsep, GZIP_FLAG)){
+			if(IS_SET_FLAG(responsep, CHUNKED_FLAG)){	
+				if(!nt_zip_inflate2(sockfd, fd))
+					goto ERROR_TRAP;
+			}else{
+				if(!nt_zip_inflate(sockfd, fd))
+					goto ERROR_TRAP;
+			}
+		}else{
+			do{
+				nread = read(sockfd, data, sizeof(data));
+				switch(nread){
+				case -1:
+					break;
+				case 0:
+					break;
+				default:
+					nwrite = write(fd, data, nread);
+					if(nwrite == -1){
+						goto ERROR_TRAP;
+					}
+					break;
+				}
+			}while(nread > 0);
+		}
+		nt_http_save_response_header(out_path, responsep);
+		result = TRUE;
+		break;
+	case 206: /* Partial content */
+		fd = open(out_path,
+			O_APPEND | O_WRONLY,
+			S_IRUSR | S_IWUSR | S_IROTH);
+		if(fd == -1){
+			result = FALSE;
+			fd = -1;
+			break;
+		}
 		do{
 			nread = read(sockfd, data, sizeof(data));
 			switch(nread){
@@ -322,27 +371,32 @@ BOOL nt_http_get(const char *url, const char *out_path,
 			default:
 				nwrite = write(fd, data, nread);
 				if(nwrite == -1){
-					close(sockfd);
-					return	-1;
+					goto ERROR_TRAP;
 				}
 				break;
 			}
 		}while(nread > 0);
-		result = (nread == 0);
+		result = TRUE;
 		break;
 	case 304: /* Not Modified */
 		result = TRUE;
 		break;
+	case 416:
+		retry = TRUE;
+		break;	
 	default:
-		result = FALSE;
+		break;
 	}/* switch */
-	
+ERROR_TRAP:	
 	if(fd > 0)
 		close(fd);
 	close(sockfd);
 	nt_socket_free(socketp);
 	nt_http_free_header(headerp);
 	nt_http_free_response_header(responsep);
+	if(retry && range)
+		return nt_http_get(url, out_path, referer, user_agent, 
+				extend_headers, FALSE);
 
 	return result;
 }
@@ -385,6 +439,7 @@ static BOOL nt_http_init_header2(
 
 	if(0 == stat(out_path, &st)){
 		headerp->fsize = st.st_size;
+		SET_FLAG(headerp, RANGE_FLAG);
 	}else{
 		headerp->fsize = -1;
 	}
