@@ -203,6 +203,16 @@ static int add_header(nt_http_response_header_tp headerp,
 			return -1;
 		headerp->content_length = num;
 	}else if(0 == strncmp("Content-Type:", line, 13)){
+	}else if(0 == strncmp("Content-Encoding:", line, 17)){
+		cptr = strstr(line+17, "gzip");
+		if(cptr){
+			SET_FLAG(headerp, GZIP_FLAG);
+		}
+	}else if(0 == strncmp("Transfer-Encoding:", line, 18)){
+		cptr = strstr(line+18, "chunked");
+		if(cptr){
+			SET_FLAG(headerp, CHUNKED_FLAG);
+		}
 	}else if(0 == strncmp("Set-Cookie:", line, 11)){
 		if(!cookiep || !host)
 			return -1;
