@@ -19,6 +19,8 @@ typedef struct tag_ctx_reslist_t
 	int res_num;
 	int cur_res;
 	int cur_res_offset;
+	int max_cur_res;
+	int max_cur_offset;
 	nt_stack_tp	selected_num_stackp;
 
 } ctx_reslist_t;
@@ -44,7 +46,6 @@ int disp_reslist(nt_window_tp wp, nt_2ch_model_tp modelp)
 	wchar_t buf[1024*3+1];
 	wchar_t *cptr;
 	nt_link_tp link_curp;
-	BOOL bottom_chk = FALSE;
 	int state;
 
 	ctxp = (ctx_reslist_tp)wp->data;
@@ -58,6 +59,68 @@ int disp_reslist(nt_window_tp wp, nt_2ch_model_tp modelp)
 
 	threadp = modelp->selected_threadp;
 
+	if(ctxp->max_cur_res < 0 || ctxp->max_cur_offset < 0){
+		wlines = wp->lines;
+		int cur_res = ctxp->res_num - 1;
+		/* get the bottom res */
+		clistp = threadp->reslistp->prev;
+		while(clistp != threadp->reslistp){
+			resp = (nt_res_tp)clistp->data;
+			if(!resp->msg_line_linkp){
+				parse_res_msg(resp, wp->cols-5);
+			}
+
+			if(resp->msg_header_line_num <= 0){
+				if(-1 == swprintf(buf, sizeof(buf)-1, 
+						L"%5d. %ls %ls %ls", 
+						resp->seq_no, resp->name,
+						resp->mail, resp->misc)){
+					resp->msg_header_line_num = 1;
+				}else{
+					len = wcslen(buf);
+					cptr = buf;
+					while(len > 0){
+						num = nt_get_wc_count_within_colmns(
+									cptr, wp->cols);
+						if(num <= 0)
+							break;
+						resp->msg_header_line_num++;
+						len -= num;
+						cptr += num;
+					}
+				}
+			}
+			wlines--;
+			if(wlines == 0){
+				ctxp->max_cur_offset = resp->msg_header_line_num;
+				ctxp->max_cur_offset += resp->msg_line_num;
+				ctxp->max_cur_res = cur_res;
+				break;
+			}
+			if(wlines <= resp->msg_line_num){
+				ctxp->max_cur_offset = resp->msg_header_line_num;
+				ctxp->max_cur_offset += resp->msg_line_num - wlines;
+				ctxp->max_cur_res = cur_res;
+				break;
+			}
+			wlines -= resp->msg_line_num;
+			if(wlines <= resp->msg_header_line_num){
+				ctxp->max_cur_offset = 
+						resp->msg_header_line_num - wlines;
+				ctxp->max_cur_res = cur_res;
+				break;
+			}
+			wlines -= resp->msg_header_line_num;
+			cur_res--;
+			if(cur_res < 0){
+				ctxp->max_cur_offset = 0;
+				ctxp->max_cur_res = 0;
+				break;
+			}
+			clistp = clistp->prev;
+		}
+	}
+
 	switch(ch){
 	case NT_KEY_CLOSE:
 		return DISP_STATE_THREADTITLE; 
@@ -67,9 +130,8 @@ int disp_reslist(nt_window_tp wp, nt_2ch_model_tp modelp)
 		if(!threadp->reslistp)
 			break;
 		clistp = threadp->reslistp->prev;
-		ctxp->cur_res = ctxp->res_num - 1;
-		ctxp->cur_res_offset = 0;
-		bottom_chk = TRUE;
+		ctxp->cur_res = ctxp->max_cur_res;
+		ctxp->cur_res_offset = ctxp->max_cur_offset;
 		break;
 	case NT_KEY_LEFT:
 		if(!ctxp->selected_num_stackp)
@@ -95,7 +157,6 @@ int disp_reslist(nt_window_tp wp, nt_2ch_model_tp modelp)
 		clistp = threadp->reslistp;
 		for(i = 0; i < num; i++)
 			clistp = clistp->next;
-		bottom_chk = TRUE;
 		break;
 	case NT_KEY_RIGHT:
 		if(!ctxp->selected_num_stackp)
@@ -115,157 +176,235 @@ int disp_reslist(nt_window_tp wp, nt_2ch_model_tp modelp)
 		clistp = threadp->reslistp;
 		for(i = 0; i < num; i++)
 			clistp = clistp->next;
-		bottom_chk = TRUE;
 		break;
 	case NT_KEY_COMMAND1:
 		assert(wp->cmd_param);
-				if(parse_cmd1(wp->cmd_param, &state)){
-					return state;
-				}
-				num = atoi(wp->cmd_param);
-				if(0 == num)
-					break;
-				num--;
-				if(ctxp->cur_res == num)
-					break;
-				if(num >= ctxp->res_num)
-					break;
-				nptr = malloc(sizeof(int));
-				*nptr = ctxp->cur_res;
-				ctxp->cur_res = num;
-				ctxp->cur_res_offset = 0;
-				if(!threadp->reslistp){
-					free(nptr);
-					break;
-				}
-				clistp = threadp->reslistp;
-				for(i = 0; i < num; i++)
-					clistp = clistp->next;
-				nt_stack_push(ctxp->selected_num_stackp, nptr);
-				bottom_chk = TRUE;
+		if(parse_cmd1(wp->cmd_param, &state)){
+			return state;
+		}
+		num = atoi(wp->cmd_param);
+		if(0 == num)
+			break;
+		num--;
+		if(ctxp->cur_res == num)
+			break;
+		if(num >= ctxp->res_num)
+			break;
+		nptr = malloc(sizeof(int));
+		*nptr = ctxp->cur_res;
+		ctxp->cur_res = num;
+		ctxp->cur_res_offset = 0;
+		if(!threadp->reslistp){
+			free(nptr);
+			break;
+		}
+		clistp = threadp->reslistp;
+		for(i = 0; i < num; i++)
+			clistp = clistp->next;
+		nt_stack_push(ctxp->selected_num_stackp, nptr);
+		//bottom_chk = TRUE;
+		break;
+	case NT_KEY_UP:
+		ctxp->cur_res_offset--;
+		if(0 <= ctxp->cur_res_offset){
+			break;
+		}
+		if(0 >= ctxp->cur_res){
+			ctxp->cur_res_offset = 0;
+			ctxp->cur_res = 0;
+			break;
+		}
+		ctxp->cur_res--;
+		clistp = threadp->reslistp;
+		for(i = 0; i < ctxp->res_num; i++){
+			if(i == ctxp->cur_res)
 				break;
-			case NT_KEY_UP:
-				ctxp->cur_res_offset--;
-				if(0 > ctxp->cur_res_offset){
-					if(ctxp->cur_res > 0){
-						ctxp->cur_res--;
-						ctxp->cur_res_offset = -1;
-					}else{
-						ctxp->cur_res_offset = 0;
-					}
-				}
-				break;
-			case NT_KEY_DOWN:
-				ctxp->cur_res_offset++;
-				clistp = threadp->reslistp;
-				for(i = 0; i < ctxp->res_num; i++){
-					if(i == ctxp->cur_res)
+			clistp = clistp->next;
+		}
+		if(i == ctxp->res_num)
+			break;
+		resp = (nt_res_tp)clistp->data;
+		if(!resp->msg_line_linkp){
+			parse_res_msg(resp, wp->cols-5);
+		}
+		if(resp->msg_header_line_num <= 0){
+			if(-1 == swprintf(buf, sizeof(buf)-1, 
+					L"%5d. %ls %ls %ls", 
+					resp->seq_no, resp->name,
+					resp->mail, resp->misc)){
+				resp->msg_header_line_num = 1;
+			}else{
+				len = wcslen(buf);
+				cptr = buf;
+				while(len > 0){
+					num = nt_get_wc_count_within_colmns(
+								cptr, wp->cols);
+					if(num <= 0)
 						break;
-					clistp = clistp->next;
+					resp->msg_header_line_num++;
+					len -= num;
+					cptr += num;
 				}
-				if(i == ctxp->res_num)
-					break;
-				bottom_chk = TRUE;
-				break;
-			case NT_KEY_PAGEUP:
-				if(ctxp->cur_res_offset > wp->lines){
-					ctxp->cur_res_offset -= wp->lines;
-					break;
-				}
-				clistp = threadp->reslistp;
-				for(i = 0; i < ctxp->res_num; i++){
-					if(i == ctxp->cur_res)
-						break;
-					clistp = clistp->next;
-				}
-				if(i == ctxp->res_num)
-					break;
-				if(i == 0){
-					ctxp->cur_res_offset = 0;
-					ctxp->cur_res = 0;
-					break;
-				}
-				wlines = wp->lines - ctxp->cur_res_offset;
-				for( ; i > 0; i--){
-					resp = (nt_res_tp)clistp->data;
-					if(!resp->msg_line_linkp){
-						parse_res_msg(resp, wp->cols-5);
-					}
-					num = resp->msg_line_num + 2;
-					if(num > wlines){
-						ctxp->cur_res = i;
-						ctxp->cur_res_offset = num - wlines;
-						break;
-					}
-					wlines -= num;
-					clistp = clistp->prev;
-				}
-				if(i == 0){
-					ctxp->cur_res = 0;
-					ctxp->cur_res_offset = 0;
-				}
-				break;
-			case NT_KEY_PAGEDOWN:
-				clistp = threadp->reslistp;
-				for(i = 0; i < ctxp->res_num; i++){
-					if(i == ctxp->cur_res)
-						break;
-					clistp = clistp->next;
-				}
-				if(i == ctxp->res_num)
-					break;
-				
-				ctxp->cur_res_offset += wp->lines;
-				for( ; i < ctxp->res_num; i++){
-					resp = (nt_res_tp)clistp->data;
-					if(!resp->msg_line_linkp){
-						parse_res_msg(resp, wp->cols-5);
-					}
-					num = resp->msg_line_num + 2;
-					if(num > ctxp->cur_res_offset)
-						break;
-
-					ctxp->cur_res++;
-					ctxp->cur_res_offset -= num;
-
-					clistp = clistp->next;
-				}
-				bottom_chk = TRUE;
-				break;
-			}
-			if(bottom_chk){
-				/* Check the botton line. */
-				wlines = wp->lines + ctxp->cur_res_offset;
-				i = ctxp->cur_res;
-				for( ; i < ctxp->res_num; i++){
-					resp = (nt_res_tp)clistp->data;
-					if(!resp->msg_line_linkp){
-						parse_res_msg(resp, wp->cols-5);
-					}
-					num = resp->msg_line_num + 2;
-					wlines -= num;
-					if(wlines <= 0)
-						break;
-					clistp = clistp->next;
-				}
-				/* Adjust the bottom line. */
-				if(i == ctxp->res_num){
-					ctxp->cur_res_offset = wp->lines;
-					ctxp->cur_res = ctxp->res_num - 1;
-					for( ; i > 0; i--){
-						resp = (nt_res_tp)clistp->data;
-						if(!resp->msg_line_linkp){
-							parse_res_msg(resp, wp->cols-5);
-				}
-				num = resp->msg_line_num + 2;
-				if(num > ctxp->cur_res_offset)
-					break;
-
-				ctxp->cur_res--;
-				ctxp->cur_res_offset -= num;
-				clistp = clistp->prev;
 			}
 		}
+		num = resp->msg_header_line_num;
+		num += resp->msg_line_num;
+		ctxp->cur_res_offset = num;
+		break;
+	case NT_KEY_DOWN:
+		ctxp->cur_res_offset++;
+		clistp = threadp->reslistp;
+		for(i = 0; i < ctxp->res_num; i++){
+			if(i == ctxp->cur_res)
+				break;
+			clistp = clistp->next;
+		}
+		if(i == ctxp->res_num)
+			break;
+		resp = (nt_res_tp)clistp->data;
+		if(!resp->msg_line_linkp){
+			parse_res_msg(resp, wp->cols-5);
+		}
+		if(resp->msg_header_line_num <= 0){
+			if(-1 == swprintf(buf, sizeof(buf)-1, 
+					L"%5d. %ls %ls %ls", 
+					resp->seq_no, resp->name,
+					resp->mail, resp->misc)){
+				resp->msg_header_line_num = 1;
+			}else{
+				len = wcslen(buf);
+				cptr = buf;
+				while(len > 0){
+					num = nt_get_wc_count_within_colmns(
+								cptr, wp->cols);
+					if(num <= 0)
+						break;
+					resp->msg_header_line_num++;
+					len -= num;
+					cptr += num;
+				}
+			}
+		}
+		num = resp->msg_header_line_num;
+		num += resp->msg_line_num + 1;
+		if(ctxp->cur_res_offset >= num){
+			ctxp->cur_res++;
+			ctxp->cur_res_offset = 0;
+		}
+		break;
+	case NT_KEY_PAGEUP:
+		ctxp->cur_res_offset -= wp->lines;
+		if(ctxp->cur_res_offset >= 0){
+			break;
+		}
+		ctxp->cur_res--;
+		clistp = threadp->reslistp;
+		for(i = 0; i < ctxp->res_num; i++){
+			if(i == ctxp->cur_res)
+				break;
+			clistp = clistp->next;
+		}
+		if(i == ctxp->res_num){
+			ctxp->cur_res_offset = 0;
+			break;
+		}
+		for( ; i > 0; i--){
+			resp = (nt_res_tp)clistp->data;
+			if(!resp->msg_line_linkp){
+				parse_res_msg(resp, wp->cols-5);
+			}
+			if(resp->msg_header_line_num <= 0){
+				if(-1 == swprintf(buf, sizeof(buf)-1, 
+						L"%5d. %ls %ls %ls", 
+						resp->seq_no, resp->name,
+						resp->mail, resp->misc)){
+					resp->msg_header_line_num = 1;
+				}else{
+					len = wcslen(buf);
+					cptr = buf;
+					while(len > 0){
+						num = nt_get_wc_count_within_colmns(
+									cptr, wp->cols);
+						if(num <= 0)
+							break;
+						resp->msg_header_line_num++;
+						len -= num;
+						cptr += num;
+					}
+				}
+			}
+			num = resp->msg_header_line_num;
+			num += resp->msg_line_num;
+			if(ctxp->cur_res_offset + num < 0){
+				ctxp->cur_res_offset += num;
+				ctxp->cur_res--;
+				clistp = clistp->prev;
+				continue;
+			}
+			ctxp->cur_res_offset += num;
+			break;
+		}
+		if(i == 0){
+			ctxp->cur_res = 0;
+			ctxp->cur_res_offset = 0;
+		}
+		break;
+	case NT_KEY_PAGEDOWN:
+		clistp = threadp->reslistp;
+		if(0 > ctxp->cur_res)
+			ctxp->cur_res = 0;
+		for(i = 0; i < ctxp->res_num; i++){
+			if(i == ctxp->cur_res)
+				break;
+			clistp = clistp->next;
+		}
+		if(i == ctxp->res_num)
+			break;
+				
+		ctxp->cur_res_offset += wp->lines;
+		for( ; i < ctxp->res_num; i++){
+			resp = (nt_res_tp)clistp->data;
+			if(!resp->msg_line_linkp){
+				parse_res_msg(resp, wp->cols-5);
+			}
+			if(resp->msg_header_line_num <= 0){
+				if(-1 == swprintf(buf, sizeof(buf)-1, 
+						L"%5d. %ls %ls %ls", 
+						resp->seq_no, resp->name,
+						resp->mail, resp->misc)){
+					resp->msg_header_line_num = 1;
+				}else{
+					len = wcslen(buf);
+					cptr = buf;
+					while(len > 0){
+						num = nt_get_wc_count_within_colmns(
+									cptr, wp->cols);
+						if(num <= 0)
+							break;
+						resp->msg_header_line_num++;
+						len -= num;
+						cptr += num;
+					}
+				}
+			}
+			num = resp->msg_header_line_num;
+			num += resp->msg_line_num + 1;
+			if(num > ctxp->cur_res_offset){
+				break;
+			}
+			ctxp->cur_res++;
+			ctxp->cur_res_offset -= num;
+
+			clistp = clistp->next;
+		}
+		break;
+	}/* end switch */
+
+	if(ctxp->cur_res > ctxp->max_cur_res ||
+		(ctxp->cur_res == ctxp->max_cur_res && 
+		ctxp->cur_res_offset > ctxp->max_cur_offset)){
+		ctxp->cur_res = ctxp->max_cur_res;
+		ctxp->cur_res_offset = ctxp->max_cur_offset;
 	}
 
 	clistp = threadp->reslistp;
@@ -328,24 +467,6 @@ int disp_reslist(nt_window_tp wp, nt_2ch_model_tp modelp)
 		}
 		if(0 == resp->msg_line_num)
 			continue;
-		if(ctxp->cur_res == i){
-			if(ctxp->cur_res_offset > resp->msg_line_num){
-				ctxp->cur_res++;
-				ctxp->cur_res_offset = 0;
-				rows = 0;
-				res_offset = 0;
-				i--;
-				continue;
-			}
-		}
-		if(ctxp->cur_res_offset == -1){
-			ctxp->cur_res_offset = resp->msg_line_num;
-			rows = 0;
-			res_offset = 0;
-			i = -1;
-			clistp = threadp->reslistp;
-			continue;
-		}
 
 		link_curp = resp->msg_line_linkp;
 		do{
@@ -403,9 +524,12 @@ static ctx_reslist_tp init_context(nt_2ch_model_tp modelp)
 		return NULL;
 
 	threadp = modelp->selected_threadp;
-	ctxp->res_num = nt_link_num(threadp->reslistp);
+	if(threadp->reslistp)
+		ctxp->res_num = nt_link_num(threadp->reslistp);
 	ctxp->cur_res = 0;
 	ctxp->cur_res_offset = 0;
+	ctxp->max_cur_res = -1;
+	ctxp->max_cur_offset = -1;
 	ctxp->selected_num_stackp = nt_stack_alloc();
 
 	return ctxp;
