@@ -86,14 +86,15 @@ void* nt_link_get(nt_link_tp link)
 	return link->data;
 }
 
-nt_link_tp nt_link_find(nt_link_tp link, void *data)
+nt_link_tp nt_link_find(nt_link_tp link, void *data, 
+					nt_compare_tp comp_func)
 {
 	nt_link_tp ptr = link;
 	do{
-		if(ptr->data  == data){
+		if((comp_func)(ptr->data, data)){
 			return ptr;
 		}
-		ptr++;
+		ptr = ptr->next;
 	}while(ptr != link);
 
 	return NULL;
@@ -105,7 +106,23 @@ nt_link_tp nt_link_next(nt_link_tp link)
 	return link->next;
 }
 
-nt_key_value_tp nt_kay_vlalue_alloc(char *key, char *value)
+void nt_all_link_free(nt_link_tp ptr, nt_memfree_tp free_func)
+{
+	nt_link_tp linkp;
+	nt_link_tp tmp_linkp;
+	assert(ptr);
+
+	linkp = ptr;
+	do{
+		if(linkp->data && free_func)
+			(free_func)(linkp->data);
+		tmp_linkp = linkp;	
+		linkp = linkp->next;
+		free(tmp_linkp);
+	}while(ptr != linkp);
+}
+
+nt_key_value_tp nt_key_value_alloc(char *key, char *value)
 {
 	nt_key_value_tp ptr = malloc(sizeof(nt_key_value_t));
 	if(ptr == NULL)
@@ -252,7 +269,6 @@ nt_stack_tp nt_stack_alloc()
 
 void nt_stack_free(nt_stack_tp ptr, nt_memfree_tp free_func)
 {
-	nt_link_tp linkp;
 	assert(ptr);
 
 	if(!free_func){
@@ -260,18 +276,20 @@ void nt_stack_free(nt_stack_tp ptr, nt_memfree_tp free_func)
 		return;
 	}
 	if(ptr->linkp){
-		linkp = ptr->linkp;
+		nt_all_link_free(ptr->linkp, free_func);
+		/*linkp = ptr->linkp;
 		do{
 			(free_func)(linkp->data);
 			linkp = linkp->next;
-		}while(linkp != ptr->linkp);
+		}while(linkp != ptr->linkp);*/
 	}
 	if(ptr->unlinkedp){
-		linkp = ptr->unlinkedp;
+		nt_all_link_free(ptr->unlinkedp, free_func);
+		/*linkp = ptr->unlinkedp;
 		do{
 			(free_func)(linkp->data);
 			linkp = linkp->next;
-		}while(linkp != ptr->unlinkedp);
+		}while(linkp != ptr->unlinkedp);*/
 	}
 	free(ptr);	
 }
